﻿Public Class IntelHex

#Region " Declarations "
  Private Const _DEFAULT_PAGE_SIZE As Integer = &H4000
  Private Const _DEFAULT_ADDRESS As Short = &H4000
  Private Const _DATA_RECORD_TYPE As Integer = 0
  Private Const _EXTENDED_LINEAR_ADDRESS_RECORD_TYPE As Integer = 2
  Private Const _LINE_DATA_LENGTH As Integer = 32
  Private _fileName As String
  Private _outfile As IO.StreamWriter
  Private _bytesRemainingInPage As Integer
  Private _dataStarted As Boolean
  Private _page As Short
  Private _address As UInt16
  Private _buffer(_LINE_DATA_LENGTH - 1) As Byte
  Private _index As Integer
  Private _bytesWritten As Integer
#End Region

#Region " Constructors/Teardown "
  Public Sub New(ByVal fileName As String)
    _fileName = fileName
  End Sub
#End Region

#Region " Public Properties "
  Public ReadOnly Property BytesWritten() As Integer
    Get
      Return _bytesWritten
    End Get
  End Property
#End Region

#Region " Public Methods "
  Public Sub Create()
    _outfile = New IO.StreamWriter(_fileName, False)
    _address = _DEFAULT_ADDRESS
    _bytesRemainingInPage = _DEFAULT_PAGE_SIZE
    _bytesWritten = 0
  End Sub

  Public Sub Write(ByVal b As Byte)
    If Not _dataStarted Then
      'Write the extended linear address record first
      _WriteExtendedLinearAddressRecord(_page)

      _dataStarted = True
    End If

    'Buffer this byte to be written
    _buffer(_index) = b
    _bytesRemainingInPage -= 1
    _index += 1
    _bytesWritten += 1

    If _index = _LINE_DATA_LENGTH Then
      'Write this line out and flush the buffer
      _WriteRecord(_DATA_RECORD_TYPE, _address, _buffer)
      _address += Convert.ToUInt16(_LINE_DATA_LENGTH)
      _index = 0
    End If

    If _bytesRemainingInPage = 0 Then
      _page += Convert.ToInt16(1)
      _WriteExtendedLinearAddressRecord(_page)
      _bytesRemainingInPage = _DEFAULT_PAGE_SIZE
      _address = _DEFAULT_ADDRESS
      _index = 0
    End If
  End Sub

  Public Sub Write(ByVal bytes() As Byte)
    For Each b As Byte In bytes
      Write(b)
    Next
  End Sub

  Public Sub Close()
    _outfile.Close()
    _outfile = Nothing
  End Sub
#End Region

#Region " Private Methods "
  Private Sub _WriteExtendedLinearAddressRecord(ByVal page As Short)
    Dim bytes(1) As Byte

    bytes(0) = &H0
    bytes(1) = Convert.ToByte(page)

    _WriteRecord(_EXTENDED_LINEAR_ADDRESS_RECORD_TYPE, &H0, bytes)
  End Sub

  Private Sub _WriteRecord(ByVal recordType As Short, ByVal address As UInt16, ByVal data() As Byte)
    Dim str As String = ":"

    str &= Convert.ToString(data.Length, 16).PadLeft(2, "0"c)
    str &= Convert.ToString(address, 16).PadLeft(4, "0"c)
    str &= Convert.ToString(recordType, 16).PadLeft(2, "0"c)

    For Each b As Byte In data
      str &= Convert.ToString(b, 16).PadLeft(2, "0"c)
    Next

    str &= Convert.ToString(_CalculateChecksum(recordType, address, data), 16).PadLeft(2, "0"c)
    str &= Environment.NewLine

    _outfile.Write(str.ToUpper().ToCharArray(), 0, str.Length)
  End Sub

  Private Function _CalculateChecksum(ByVal recordType As Short, ByVal address As UInt16, ByVal data() As Byte) As Short
    Dim ret As Short

    ret = Convert.ToInt16((ret + recordType) And 255)
    ret = Convert.ToInt16((ret + (address And 255)) And 255)
    ret = Convert.ToInt16((ret + (address >> 8)) And 255)
    ret = Convert.ToInt16((ret + data.Length) And 255)
    For Each s As Short In data
      ret = Convert.ToInt16((ret + s) And 255)
    Next

    ret = Convert.ToInt16((Not ret) And 255)
    ret += Convert.ToInt16(1)

    Return Convert.ToInt16(ret And 255)
  End Function
#End Region

End Class
