﻿Public Class Main

#Region " Declarations "
  Private Const _WRONG_NUM_ARGUMENTS As Integer = 1
  Private Const _UNKNOWN_EXCEPTION As Integer = 2

  Private Const _NUM_ARGUMENTS As Integer = 2

  Private Const _APP_HEADER_LENGTH As Integer = 128
#End Region

#Region " Public Methods "
  Public Shared Sub Main(ByVal args() As String)
    'Validate correct number of arguments
    If args.Length <> _NUM_ARGUMENTS Then
      _DisplayUsage(_WRONG_NUM_ARGUMENTS, "Invalid number of arguments!")
      Exit Sub
    End If

    Try
      'Read input file
      Dim infile As New IO.FileStream(args(0), IO.FileMode.Open)
      Dim ininfo As New IO.FileInfo(args(0))

      Dim inputData(Convert.ToInt32(ininfo.Length - 1)) As Byte
      infile.Read(inputData, 0, Convert.ToInt32(ininfo.Length))
      infile.Close()

      'Write output file
      Dim outfile As New IntelHex(args(1) & ".hex")
      outfile.Create()

      'Write application header and default jump to _JForceCmdNoChar
      outfile.Write(New Byte() {&H80, &HF, &H0, &H0, &H0, &H0})
      outfile.Write(New Byte() {&H80, &H12, &H1, &H4})
      outfile.Write(New Byte() {&H80, &H21, &H1})
      outfile.Write(New Byte() {&H80, &H31, &HA4})
      outfile.Write(New Byte() {&H80, &H48})

      'Write application name
      Dim appName As String = args(1).ToUpper()
      If appName.Length > 8 Then appName.Substring(0, 8)
      appName = appName.PadRight(8, " "c)
      For Each c As Char In appName
        outfile.Write(Convert.ToByte(c))
      Next

      'Write number of pages
      Dim numPages As Byte = Convert.ToByte(Math.Ceiling((inputData.Length + _APP_HEADER_LENGTH) / 16384.0))
      outfile.Write(New Byte() {&H80, &H81, numPages})

      outfile.Write(New Byte() {&H80, &H90})
      outfile.Write(New Byte() {&H3, &H26, &H9, &H4, &H4, &H6F, &H1B, &H80})
      outfile.Write(New Byte() {&H2, &HD, &H40})
      outfile.Write(New Byte() {&HA1, &H6B, &H99, &HF6, &H59, &HBC, &H67, &HF5})
      outfile.Write(New Byte() {&H85, &H9C, &H9, &H6C, &HF, &HB4, &H3, &H9B})
      outfile.Write(New Byte() {&HC9, &H3, &H32, &H2C, &HE0, &H3, &H20, &HE3})
      outfile.Write(New Byte() {&H2C, &HF4, &H2D, &H73, &HB4, &H27, &HC4, &HA0})
      outfile.Write(New Byte() {&H72, &H54, &HB9, &HEA, &H7C, &H3B, &HAA, &H16})
      outfile.Write(New Byte() {&HF6, &H77, &H83, &H7A, &HEE, &H1A, &HD4, &H42})
      outfile.Write(New Byte() {&H4C, &H6B, &H8B, &H13, &H1F, &HBB, &H93, &H8B})
      outfile.Write(New Byte() {&HFC, &H19, &H1C, &H3C, &HEC, &H4D, &HE5, &H75})
      outfile.Write(New Byte() {&H80, &H7F, &H0, &H0, &H0, &H0})

      'Write our dummy jump to _JForceCmdNoChar
      outfile.Write(New Byte() {&HEF, &H27, &H40})

      'Pad the rest of the header
      For i As Integer = 0 To (_APP_HEADER_LENGTH - outfile.BytesWritten) - 1
        outfile.Write(&H0)
      Next

      'Write out the data
      outfile.Write(inputData)

      outfile.Close()
    Catch ex As Exception
      _DisplayUsage(_UNKNOWN_EXCEPTION, ex.Message)
      Exit Sub
    End Try
  End Sub
#End Region

#Region " Private Methods "
  Private Shared Sub _DisplayUsage(ByVal exitCode As Integer, ByVal msg As String)
    Console.WriteLine("To8XK " & My.Application.Info.Version.ToString(3))
    Console.WriteLine("------------------------------------------------------")
    Console.WriteLine("Usage: To8XK <BIN file> <Intel Hex file name (no extension)>")
    Console.WriteLine("Converts a binary image to an Intel Hex file to be signed")
    Console.WriteLine(" into an 83+/84+ series 8XK Flash application.")
    Console.WriteLine()
    Console.WriteLine("Examples: To8XK data.bin MYTEST")
    Console.WriteLine("  Creates an Intel Hex file called MYTEST.hex which contains")
    Console.WriteLine("   the data for a Flash application called MYTEST.")
    Console.WriteLine()
    Console.Write("ERROR: " & msg)

    Environment.ExitCode = exitCode
  End Sub
#End Region

End Class
